use Test::More tests => 1;
use strict;

use DE_EPAGES::Core::API::Constants;
use DE_EPAGES::Core::API::File qw(SetAccessRights GetInfoDirectory);
use DE_EPAGES::Core::API::Script qw(RunScript);
use DE_EPAGES::Core::API::SystemCommand qw(SystemCommand);

RunScript( Message => 'long', Sub => sub {
    SKIP: {
        skip(sprintf('User \'%s\' can\'t set file permissions, only \'root\' can do this.', (getpwuid $<)[0]), 1) if $< != 0;

        _InvalidateAccessRights($ENV{EPAGES_CONFIG});
        _FixAccessRightsBySystemCommand('admin');
        my $AccessRightsBySystemCommand = _GetAccessRights($ENV{EPAGES_CONFIG});

        _InvalidateAccessRights($ENV{EPAGES_CONFIG});
        _FixAccessRightsByPerlFunction($ENV{EPAGES_CONFIG});
        my $AccessRightsByPerlFunction = _GetAccessRights($ENV{EPAGES_CONFIG});

        is_deeply($AccessRightsBySystemCommand, $AccessRightsByPerlFunction,
                  'comparing access rights set by SetAccessRights and SystemCommand');
    }
});

sub _InvalidateAccessRights {
    my ($Dir) = @_;
    SystemCommand("chown -R root:root $Dir; chmod -R 777 $Dir");
}

sub _GetAccessRights {
    my ($Dir) = @_;
    my $hInfo = GetInfoDirectory( $Dir, { recursive => 1 } );

    my %AccessRights;
    foreach my $File ((@{$hInfo->{Directories}}, @{$hInfo->{Files}}, @{$hInfo->{Others}})) {
        $AccessRights{$File} = join(' ', (stat $File)[2,4,5]);
    }
    return \%AccessRights;
}

sub _FixAccessRightsBySystemCommand {
    my ($Dir) = @_;
    SystemCommand("/etc/init.d/epages6 perm $Dir");
}

sub _FixAccessRightsByPerlFunction {
    my ($Dir) = @_;
    my $hInfo = GetInfoDirectory( $Dir, { recursive => 1 } );

    foreach my $File ((@{$hInfo->{Directories}}, @{$hInfo->{Files}}, @{$hInfo->{Others}})) {
        SetAccessRights($File);
    }
}
