#========================================================================================
# package      DE_EPAGES::Order::t::unit::LineItemContainerData
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Order::API::LineItemContainerData
#========================================================================================
package DE_EPAGES::Order::t::unit::LineItemContainerData;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Order::API::LineItemContainerData;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockery;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_getLineItemContainerData_NormalProduct();
    $self->test_getLineItemContainerData_DownloadableProduct();
    $self->test_getLineItemContainerData_NoLineItemContainer();

    $self->test_getShippingData_NormalProduct();
    $self->test_getShippingData_DownloadableProduct();
    $self->test_getShippingData_NoLineItemContainer();

    $self->test_getPaymentData();
    $self->test_getPaymentData_NoLineItemContainer();

    done_testing();
}



sub test_getLineItemContainerData_NormalProduct {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createObjects_NormalProduct();
    my $LineItemContainerData = $Mockery->get('SUT');

    my $LanguageID = '2';
    my $LocaleID = 'en_GB';
    my $IncludeTaxInLineItem = 'false';
    my $hLineItemContainerData = $LineItemContainerData->getLineItemContainerData($LanguageID, $LocaleID, $IncludeTaxInLineItem);

    my $hExpectedLineItemContainerData = $self->_createExpectedLineItemContainerData();
    is_deeply($hLineItemContainerData, $hExpectedLineItemContainerData, "$SubTestName - test if expected LineItemContainer data was returned.");
}



sub test_getLineItemContainerData_DownloadableProduct {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createObjects_DownloadableProduct();
    my $LineItemContainerData = $Mockery->get('SUT');

    my $LanguageID = '2';
    my $LocaleID = 'en_GB';
    my $hLineItemContainerData = $LineItemContainerData->getLineItemContainerData($LanguageID, $LocaleID);

    my $hExpectedLineItemContainerData = $self->_createExpectedLineItemContainerData_DownloadableProduct();
    is_deeply($hLineItemContainerData, $hExpectedLineItemContainerData, "$SubTestName - test if expected LineItemContainer data was returned.");
}



sub test_getLineItemContainerData_NoLineItemContainer {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsLineItemContainerIncluded = 0;
    my $Mockery = $self->_createObjects_NormalProduct($IsLineItemContainerIncluded);
    my $LineItemContainerData = $Mockery->get('SUT');

    my $LanguageID = '2';
    my $LocaleID = 'en_GB';
    my $hLineItemContainerData = $LineItemContainerData->getLineItemContainerData($LanguageID, $LocaleID);

    is_deeply($hLineItemContainerData, undef, "$SubTestName - test if only undef was returned.");
}



sub test_getShippingData_NormalProduct {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createObjects_NormalProduct();
    my $LineItemContainerData = $Mockery->get('SUT');

    my $LanguageID = '2';
    my $LocaleID = 'en_GB';
    my $hShippingData = $LineItemContainerData->getShippingData($LanguageID, $LocaleID);

    my $hExpectedShippingData = $self->_createExpectedShippingData();
    is_deeply($hShippingData, $hExpectedShippingData, "$SubTestName - test if expected shipping data was returned.");
}



sub test_getShippingData_DownloadableProduct {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createObjects_DownloadableProduct();
    my $LineItemContainerData = $Mockery->get('SUT');

    my $LanguageID = '2';
    my $LocaleID = 'en_GB';
    my $hShippingData = $LineItemContainerData->getShippingData($LanguageID, $LocaleID);

    is_deeply($hShippingData, undef, "$SubTestName - test if expected shipping data was returned.");
}



sub test_getShippingData_NoLineItemContainer {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsLineItemContainerIncluded = 0;
    my $Mockery = $self->_createObjects_NormalProduct($IsLineItemContainerIncluded);
    my $LineItemContainerData = $Mockery->get('SUT');

    my $LanguageID = '2';
    my $LocaleID = 'en_GB';
    my $hShippingData = $LineItemContainerData->getShippingData($LanguageID, $LocaleID);

    is_deeply($hShippingData, undef, "$SubTestName - test if shipping data is undef if there is no LineItemContainer.");
}



sub test_getPaymentData {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createObjects_NormalProduct();
    my $LineItemContainerData = $Mockery->get('SUT');

    my $LanguageID = '2';
    my $LocaleID = 'en_GB';
    my $hPaymentData = $LineItemContainerData->getPaymentData($LanguageID, $LocaleID);

    my $hExpectedPaymentData = $self->_createExpectedPaymentData();
    is_deeply($hPaymentData, $hExpectedPaymentData, "$SubTestName - test if expected payment data was returned.");
}



sub test_getPaymentData_NoLineItemContainer {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsLineItemContainerIncluded = 0;
    my $Mockery = $self->_createObjects_NormalProduct($IsLineItemContainerIncluded);
    my $LineItemContainerData = $Mockery->get('SUT');

    my $LanguageID = '2';
    my $LocaleID = 'en_GB';
    my $hPaymentData = $LineItemContainerData->getPaymentData($LanguageID, $LocaleID);

    is_deeply($hPaymentData, undef, "$SubTestName - test if payment data is undef if there is no LineItemContainer.");
}



sub _createObjects_NormalProduct {
    my $self = shift;
    my ($IsLineItemContainerIncluded) = @_;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('LineItemContainer', ['Shop', 'LineItemShipping', 'LineItemPayment'], sub {
        my ($Shop, $LineItemShipping, $LineItemPayment) = @_;
        return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', {
            'Site' => $Shop,
            'Coupon' => undef,
            'CurrencyID' => 'EUR',
            'TaxModel' => 'GROSS',
            'GrandTotal' => '123.45',
            'TotalBeforeTax' => '56',
            'TotalTax' => '23',
            'LineItemsSubTotal' => '55.2',
            'Shipping' => $LineItemShipping,
            'Payment' => $LineItemPayment,
        });
    });

    my $DeliveryWeight = 9.78; ## no critic (ProhibitMagicNumbers)
    $Mockery->extend($self->_createCommonMocks($IsLineItemContainerIncluded, $DeliveryWeight));

    return $Mockery->assemble();
}



sub _createObjects_DownloadableProduct {
    my $self = shift;
    my ($IsLineItemContainerIncluded) = @_;
    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('LineItemContainer', ['Shop', 'LineItemPayment'], sub {
        my ($Shop, $LineItemPayment) = @_;
        return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', {
            'Site' => $Shop,
            'Coupon' => undef,
            'CurrencyID' => 'EUR',
            'TaxModel' => 'GROSS',
            'GrandTotal' => '123.45',
            'TotalBeforeTax' => '56',
            'TotalTax' => '23',
            'LineItemsSubTotal' => '55.2',
            'Shipping' => undef, # downloadable products have no LineItemShipping!
            'Payment' => $LineItemPayment,
        });
    });

    $Mockery->extend($self->_createCommonMocks($IsLineItemContainerIncluded));

    return $Mockery->assemble();
}



sub _createCommonMocks {
    my $self = shift;
    my ($IsLineItemContainerIncluded, $DeliveryWeight) = @_;

    #  Baskets that were converted to orders don't have a LineItemContainer anymore
    $IsLineItemContainerIncluded = $IsLineItemContainerIncluded // 1;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', ['BasketOrOrder', 'Formatter', 'LineItemsData'], sub {
        my ($BasketOrOrder, $Formatter, $LineItemsData) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Order::API::LineItemContainerData', [$BasketOrOrder]);
        $Mockify->addMockWithReturnValueAndParameterCheck('_createFormatter', $Formatter, [{'object'=> 'DE_EPAGES::Shop::API::Object::Shop'}, 'string']);
        $Mockify->addMockWithReturnValueAndParameterCheck('_createLineItemsData', $LineItemsData, []);
        return $Mockify->getMockObject();
    });

    $Mockery->register('BasketOrOrder', ['LineItemContainer', 'Shop'], sub {
        my ($LineItemContainer, $Shop) = @_;
         return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::Basket', {
            'Site' => $Shop,
            'LineItemContainer' => $IsLineItemContainerIncluded ? $LineItemContainer : undef,
        });
    });

    $Mockery->register('Shop', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Shop::API::Object::Shop', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('siteLocale', 'en_GB', [{'string' => 'en_GB'}]);
        return $Mockify->getMockObject();
    });

    $Mockery->register('Formatter', ['Shop'], sub {
        my ($Shop) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Shop::API::Resource::Formatter', [$Shop, 'en_GB']);
        $Mockify->addMock('quantity', sub {
            my $MockedSelf = shift;
            my ($Quantity, $Unit) = @_;
            return $Quantity;
        });
        $Mockify->addMock('price', sub {
            my $MockedSelf = shift;
            my ($Price, $Currency, $TaxModel) = @_;
            return $Price;
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('LineItemShipping', ['ShippingMethod'], sub {
        my ($ShippingMethod) = @_;
        return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemShipping', {
            'LineItemPrice' => 2.40,
            'ShippingMethod' => $ShippingMethod,
        });
    });

    $Mockery->register('LineItemsData', ['ShippingMethod'], sub {
        my ($ShippingMethod) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Order::API::LineItemsData', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('getLineItems', $self->_getLineItemsData($DeliveryWeight), ['object', 'object', 'string', 'string']);
        return $Mockify->getMockObject();
    });

    $Mockery->register('LineItemPayment', ['LineItemPaymentClass', 'PaymentMethod'], sub {
        my ($LineItemPaymentClass, $PaymentMethod) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Order::API::Object::LineItemPayment', {
            'LineItemPrice' => 3.20,
            'PaymentMethod' => $PaymentMethod,
            'PaymentErrorCode' => 'CANCELED',
        });
        $Mockify->addMockWithReturnValue('class', $LineItemPaymentClass);
        return $Mockify->getMockObject();
    });

    $Mockery->register('LineItemPaymentClass', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Class', {});
        $Mockify->addMockWithReturnValueAndParameterCheck('existsAttribute', 1, [{'string' => 'PaymentErrorCode'}]);
        return $Mockify->getMockObject();
    });

    $Mockery->register('OrderUnit', [], sub {
        return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::UnitOfMeasurement', {});
    });

    $Mockery->register('ShippingMethod', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shipping::API::Object::ShippingMethod', {
            'GUID' => '6235-43324',
            'NameOrAlias' => 'TestShipping',
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('TaxClass', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxClass', {
            'Name' => 'TaxClassName',
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('PaymentMethod', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Payment::API::Object::PaymentMethod', {
            'GUID' => '7695-5473452',
            'NameOrAlias' => 'TestPayment',
        });
        return $Mockify->getMockObject();
    });

    return $Mockery;
}

sub _getLineItemsData {
    my $self = shift;
    my ($DeliveryWeight) = @_;
    return [
          {
            'images' => [
                  {
                    'url' => 'http://image.baseurl.test.org/web/path/hotdeal',
                    'classifier' => 'HotDeal'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/large',
                    'classifier' => 'Large'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/medium',
                    'classifier' => 'Medium'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/small',
                    'classifier' => 'Small'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/thumbnail',
                    'classifier' => 'Thumbnail'
                  }
                ],
            'sku' => 'abc_12345346',
            'quantity' => 2,
            'name' => 'Product 1',
            'lineItemPrice' => '32.99',
            'productId' => '245345-32534-324234',
            'lineItemId' => '123-35-56562-324352',
            'singleItemPrice' => '32.99',
            'essentialFeatures' => 'test product',
            'taxClass' => {
                'name' => 'TaxClassName',
                'percentage' => 19
            },
            'deliveryWeight' => $DeliveryWeight,
            'links' => [
                 {
                   'rel' => 'product',
                   'href' => 'http://baseurl.test.org/api/products/245345-32534-324234'
                 }
               ]
          }
        ];
}

sub _createExpectedLineItemContainerData {
    return {
      'grandTotal' => '123.45',
      'couponLineItem' => undef,
      'productLineItems' => [
          {
            'images' => [
                  {
                    'url' => 'http://image.baseurl.test.org/web/path/hotdeal',
                    'classifier' => 'HotDeal'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/large',
                    'classifier' => 'Large'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/medium',
                    'classifier' => 'Medium'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/small',
                    'classifier' => 'Small'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/thumbnail',
                    'classifier' => 'Thumbnail'
                  },
                ],
            'sku' => 'abc_12345346',
            'quantity' => 2,
            'name' => 'Product 1',
            'lineItemPrice' => '32.99',
            'productId' => '245345-32534-324234',
            'lineItemId' => '123-35-56562-324352',
            'singleItemPrice' => '32.99',
            'essentialFeatures' => 'test product',
            'taxClass' => {
                'name' => 'TaxClassName',
                'percentage' => 19
            },
            'deliveryWeight' => '9.78',
            'links' => [
                 {
                   'rel' => 'product',
                   'href' => 'http://baseurl.test.org/api/products/245345-32534-324234'
                 }
               ]
          }
        ],
      'totalBeforeTax' => '56',
      'totalTax' => '23',
      'lineItemsSubTotal' => '55.2',
      'shippingPrice' => '2.4',
    };
}



sub _createExpectedLineItemContainerData_DownloadableProduct {
    return {
      'grandTotal' => '123.45',
      'couponLineItem' => undef,
      'productLineItems' => [
          {
            'images' => [
                  {
                    'url' => 'http://image.baseurl.test.org/web/path/hotdeal',
                    'classifier' => 'HotDeal'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/large',
                    'classifier' => 'Large'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/medium',
                    'classifier' => 'Medium'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/small',
                    'classifier' => 'Small'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/thumbnail',
                    'classifier' => 'Thumbnail'
                  }
                ],
            'sku' => 'abc_12345346',
            'quantity' => 2,
            'name' => 'Product 1',
            'lineItemPrice' => '32.99',
            'productId' => '245345-32534-324234',
            'lineItemId' => '123-35-56562-324352',
            'singleItemPrice' => '32.99',
            'essentialFeatures' => 'test product',
            'taxClass' => {
                'name' => 'TaxClassName',
                'percentage' => 19
            },
            'deliveryWeight' => undef,
            'links' => [
                 {
                   'rel' => 'product',
                   'href' => 'http://baseurl.test.org/api/products/245345-32534-324234'
                 }
               ]
          }
        ],
      'totalBeforeTax' => '56',
      'totalTax' => '23',
      'lineItemsSubTotal' => '55.2',
      'shippingPrice' => '0'
    };
}



sub _createExpectedShippingData {
    return {
        'shippingMethod' => {
            'name' => 'TestShipping',
            'id' => '6235-43324'
        },
        'price' => '2.4'
    };
}



sub _createExpectedPaymentData {
    return {
        'paymentMethod' => {
            'name' => 'TestPayment',
            'id' => '7695-5473452'
        },
        'price' => '3.2',
        'status' => 'CANCELED',
    };
}

__PACKAGE__->runScript();