#========================================================================================
# package      DE_EPAGES::Packlink::t::unit::AppStorePacklink
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for class AppStorePacklink.
#========================================================================================
package DE_EPAGES::Packlink::t::unit::AppStorePacklink;

use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Packlink::API::AppStorePacklink;
use DE_EPAGES::Test::API::TestTools qw ( TestExistsMethod );
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::MockifyServlet;
use Test::Exception;
use Test::MockObject;
use Test::MockObject::Extends;
use Test::More;

use strict;

use constant APPID => '1234567890';

sub test {
    my $self = shift;
    $self->check_packlink_app_validAppId_uninstallLink();
    $self->check_packlink_app_novalidAppId_installLink();
    $self->check_packlink_app_validAppId_installLink();
    $self->install_packlink_validAppId();
    $self->install_packlink_validAppId_withHookError();
    $self->install_packlink_invalidAppId();
    $self->uninstall_packlink_validAppId();
    $self->uninstall_packlink_invalidAppId();
    $self->uninstall_packlink_validAppId_withHookError();
    $self->uninstall_packlink_with_ShippingProviderMethodPacklink_positive();
    done_testing();
}

sub check_packlink_app_validAppId_uninstallLink {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsAppIdValid            = '1';
    my $GenerateInstallLink     = '0';
    my $GenerateHookError       = '0';
    my $MockedAppStorePacklink  = $self->_createAppStorePacklink( $IsAppIdValid, $GenerateInstallLink, $GenerateHookError );
    my $MockedShop              = $self->_createShop();
    is( $MockedAppStorePacklink->isPacklinkAppInstalled( $MockedShop, ), '1', "$SubTestName - Packlink App is installed." );

    return;
}

sub check_packlink_app_novalidAppId_installLink {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsAppIdValid            = '0';
    my $GenerateInstallLink     = '1';
    my $GenerateHookError       = '0';
    my $MockedAppStorePacklink  = $self->_createAppStorePacklink( $IsAppIdValid, $GenerateInstallLink, $GenerateHookError );
    my $MockedShop              = $self->_createShop();
    is( $MockedAppStorePacklink->isPacklinkAppInstalled( $MockedShop, ), '0', "$SubTestName - Packlink App is not installed." );

    return;
}


sub check_packlink_app_validAppId_installLink {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsAppIdValid            = '1';
    my $GenerateInstallLink     = '1';
    my $GenerateHookError       = '0';
    my $MockedAppStorePacklink  = $self->_createAppStorePacklink( $IsAppIdValid, $GenerateInstallLink, $GenerateHookError );
    my $MockedShop              = $self->_createShop();
    is( $MockedAppStorePacklink->isPacklinkAppInstalled( $MockedShop, ), '0', "$SubTestName - Packlink App is not installed." );

    return;
}

sub install_packlink_validAppId {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsAppIdValid            = '1';
    my $GenerateInstallLink     = '1';
    my $GenerateHookError       = '0';
    my $MockedAppStorePacklink  = $self->_createAppStorePacklink( $IsAppIdValid, $GenerateInstallLink, $GenerateHookError );
    my $MockedShop              = $self->_createShop();

    is( $MockedAppStorePacklink->installPacklinkApp( APPID, $MockedShop ),
        '1', "$SubTestName - Packlink App has been successfully installed with a valid appId." );

    return;
}

sub install_packlink_validAppId_withHookError {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsAppIdValid            = '1';
    my $GenerateInstallLink     = '1';
    my $GenerateHookError       = '1';
    my $MockedAppStorePacklink  = $self->_createAppStorePacklink( $IsAppIdValid, $GenerateInstallLink, $GenerateHookError );
    my $MockedShop              = $self->_createShop();

    is( $MockedAppStorePacklink->installPacklinkApp( APPID, $MockedShop ),
        '0', "$SubTestName - Packlink App hasn't been installed with a valid appId." );

    return;
}

sub install_packlink_invalidAppId {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsAppIdValid            = '0';
    my $GenerateInstallLink     = '1';
    my $GenerateHookError       = '0';
    my $MockedAppStorePacklink  = $self->_createAppStorePacklink( $IsAppIdValid, $GenerateInstallLink, $GenerateHookError );
    my $MockedShop              = $self->_createShop();

    is( $MockedAppStorePacklink->installPacklinkApp( APPID, $MockedShop ),
        '1', "$SubTestName - Packlink App has been successfully installed with a invalid appId." );

    return;
}

sub uninstall_packlink_validAppId {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsAppIdValid            = '1';
    my $GenerateInstallLink     = '1';
    my $GenerateHookError       = '0';
    my $MockedAppStorePacklink  = $self->_createAppStorePacklink( $IsAppIdValid, $GenerateInstallLink, $GenerateHookError );
    my $MockedShop              = $self->_createShop();

    is( $MockedAppStorePacklink->uninstallPacklinkApp( APPID, $MockedShop, $MockedShop ),
        '1', "$SubTestName - Packlink App has been successfully uninstalled with a valid appId." );

    return;
}


sub uninstall_packlink_validAppId_withHookError {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsAppIdValid           = '1';
    my $GenerateInstallLink    = '1';
    my $GenerateHookError      = '1';
    my $MockedAppStorePacklink = $self->_createAppStorePacklink( $IsAppIdValid, $GenerateInstallLink, $GenerateHookError );
    my $MockedShop             = $self->_createShop();

    is( $MockedAppStorePacklink->uninstallPacklinkApp( APPID, $MockedShop, $MockedShop ),
        '0', "$SubTestName - Packlink App hasn't been uninstalled with a valid appId." );

    return;
}

sub uninstall_packlink_invalidAppId {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsAppIdValid            = '0';
    my $GenerateInstallLink     = '1';
    my $GenerateHookError       = '0';
    my $MockedAppStorePacklink  = $self->_createAppStorePacklink( $IsAppIdValid, $GenerateInstallLink, $GenerateHookError );
    my $MockedShop              = $self->_createShop();

    is( $MockedAppStorePacklink->uninstallPacklinkApp( APPID, $MockedShop, $MockedShop ),
        '1', "$SubTestName - Packlink App has been successfully uninstalled with a invalid appId." );

    return;
}

sub uninstall_packlink_with_ShippingProviderMethodPacklink_positive {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsAppIdValid            = '1';
    my $GenerateInstallLink     = '1';
    my $GenerateHookError       = '0';
    my $MockedAppStorePacklink  = $self->_createAppStorePacklink( $IsAppIdValid, $GenerateInstallLink, $GenerateHookError );
    my $MockedShop                           = $self->_createShop();
    my $MockedShippingProviderMethodPacklink = $self->_createShippingProviderMethodPacklink();

    is( $MockedAppStorePacklink->uninstallPacklinkApp( APPID, $MockedShippingProviderMethodPacklink, $MockedShop ),
        '1', "$SubTestName - Packlink App has been successfully uninstalled." );

    return;
}

#----------------------------------------------------------------------------------------

sub _createAppStorePacklink {
    my $self = shift;
    my ( $IsValid, $GenerateInstallLink, $GenerateHookError ) = @_;

    my $aParameterList = [];
    my $MockedAppStorePacklink = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::Packlink::API::AppStorePacklink', $aParameterList );

    my $aParameterTypes = ['hashref'];
    $MockedAppStorePacklink->addMockWithReturnValueAndParameterCheck( '_createAppsClient',
        $self->_createAppsClient($GenerateInstallLink), $aParameterTypes );

    $aParameterTypes = [ 'string', 'object', 'string' ];
    $MockedAppStorePacklink->addMockWithReturnValueAndParameterCheck( '_TestObject', '', $aParameterTypes );

    $aParameterTypes = ['string'];
    $MockedAppStorePacklink->addMockWithReturnValueAndParameterCheck( '_isValid', $IsValid, $aParameterTypes );

    $aParameterTypes = [ 'string', 'string', 'string', 'string' ];
    $MockedAppStorePacklink->addMockWithReturnValueAndParameterCheck( '_LogShippingFulfillment', '', $aParameterTypes );

    $MockedAppStorePacklink->mock(
        '_TriggerHook',
        sub {
            shift;
            if( $GenerateHookError ){
                return Error(' Error TriggerHook ');
            }
            return;
        }
    );

    return $MockedAppStorePacklink->getMockObject();
}

sub _createAppsClient {
    my $self = shift;
    my ( $GenerateInstallLink ) = @_;

    my $MockedAppsClient = Test::MockObject->new();
    TestExistsMethod( 'DE_EPAGES::AppStore::API::Object::AppsClient', 'findAppIdByUrl' );
    $MockedAppsClient->mock(
        'findAppIdByUrl',
        sub {
            my $MockedSelf = shift;
            my ( $appType, $appUrl ) = @_;
            $self->{'mockedParams'}->{'AppType'} = $appType;
            $self->{'mockedParams'}->{'Domain'}  = $appUrl;

            return APPID;
        }
    );

    $MockedAppsClient->mock(
        'getAppLinks',
        sub {
            my $MockedSelf = shift;
            my ($appId) = @_;

            my $relLink = 'uninstallApp' ;
            if ($GenerateInstallLink) {
                $relLink = 'installApp';
            }

            my $hLinks= { 'links' => [ { 'rel' => $relLink } ] };

            return $hLinks;
        }
    );

    return $MockedAppsClient;
}

sub _createShop {
    my $self = shift;

    my $hAttributes = { 'ShopTypeRef' => $self->_createShopTypeRef() };
    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hAttributes );


    $Shop->mock('instanceOf', sub {
        shift;
        return 1;
    });


    return $Shop;
}

sub _createShopTypeRef {
    my $self = shift;

    my $hAttributes = { 'Alias' => 'Demo' };
    my $ShopTypeRef =
      DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::ShopTypeRef', $hAttributes );

    return $ShopTypeRef;
}

sub _createShippingProviderMethodPacklink {
    my $self = shift;

    my $hAttributes = {};
    my $MockedShippingProviderMethodPacklink = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Packlink::API::Object::ShippingProviderMethodPacklink', $hAttributes );

    $MockedShippingProviderMethodPacklink->mock('instanceOf', sub {
        shift;
        return 0;
    });

    return $MockedShippingProviderMethodPacklink;
}

__PACKAGE__->runScript();

1;
